if (${CMAKE_SYSTEM_PROCESSOR} MATCHES "64")
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} \
  -m64")
else()
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} \
  -m32")
endif()

if(${CMAKE_C_COMPILER_ID} MATCHES "GNU")
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} \
  ")
  set(CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} \
  -O3 -march=native -flto \
  ")
  set(CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} \
  -g -ggdb -Wall -Wextra \
  -ftrapv \
  ")
  set(CMAKE_C_FLAGS_PROFILING "-pg -g")
  set(CMAKE_C_FLAGS_COVERAGE "--coverage -g")

  add_definitions(-D_CC_GNU_)

elseif(${CMAKE_C_COMPILER_ID} MATCHES "Intel")
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} \
  ")
  set(CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} \
  ")
  set(CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} \
  ")
  set(CMAKE_C_FLAGS_PROFILING "-p -traceback -g")
  set(CMAKE_C_FLAGS_COVERAGE "-prof-gen=srcpos -traceback -g")

  add_definitions(-D_CC_INTEL_)

elseif(${CMAKE_C_COMPILER_ID} MATCHES "PGI")
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} \
  ")
  set(CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} \
  -O3 \
  ")
  set(CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} \
  -g -Mbounds -traceback \
  ")
  set(CMAKE_C_FLAGS_PROFILING "-Mprof -traceback -g -pg")

  add_definitions(-D_CC_PGI_)

endif()

option(ENABLE_OMP "Enable OpenMP support" OFF)
if(ENABLE_OMP)
  find_package(OpenMP REQUIRED)
  add_definitions(-D_CC_PAR_OMP_)
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} ${OpenMP_C_FLAGS}")
  #"
endif()

option(ENABLE_MPI "Enable MPI support" OFF)
if(ENABLE_MPI)
  find_package(MPI REQUIRED)
  add_definitions(-D_CC_PAR_MPI_)
  target_include_directories(${PROJECT_NAME} PUBLIC ${MPI_C_INCLUDE_PATH})
  target_link_libraries(${PROJECT_NAME} ${MPI_C_LIBRARIES})
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} ${MPI_C_FLAGS}")
  #"
endif()
