################################################################################
# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

option(ENABLE_LAPACK "Enable the LAPACK library?" OFF)
if (ENABLE_LAPACK)
  find_package(LAPACK REQUIRED)
  if(LAPACK_FOUND)
    target_link_libraries(${PROJECT_NAME} ${LAPACK_LIBRARIES})
    add_definitions(-D_ENABLE_LAPACK_='${BLA_VENDOR}')
  endif(LAPACK_FOUND)
endif()

option(ENABLE_SILO "Enable the output with the Silo library?" OFF)
if (ENABLE_SILO)
  # From: https://github.com/Kitware/ParaView/blob/master/CMake/FindSILO.cmake
  find_package(SILO REQUIRED)
  if(SILO_FOUND)
    target_link_libraries(${PROJECT_NAME} ${SILO_LIBRARIES})
    target_include_directories(${PROJECT_NAME} PUBLIC ${SILO_INCLUDE_DIR})
    add_definitions(-D_ENABLE_SILO_)
  else()
  endif(SILO_FOUND)
endif()

option(ENABLE_HDF5 "Enable the output with the hdf5 library?" OFF)
if (ENABLE_HDF5)
  find_package(HDF5 REQUIRED Fortran)
  if(HDF5_FOUND)
    target_link_libraries(${PROJECT_NAME} ${HDF5_LIBRARIES} ${HDF5_Fortran_LIBRARIES})
    target_include_directories(${PROJECT_NAME} PUBLIC ${HDF5_INCLUDE_DIRS})
    add_definitions(-D_ENABLE_HDF5_)
  else()
  endif(HDF5_FOUND)
endif()

option(ENABLE_PETSc "Enable PETSc library?" OFF)
if (ENABLE_PETSc)
  find_package(PETSc REQUIRED)
  if(PETSc_FOUND)
    target_link_libraries(${PROJECT_NAME} ${PETSC_LIBRARIES})
    target_include_directories(${PROJECT_NAME} PUBLIC ${PETSC_INCLUDES})
    add_definitions(${PETSC_DEFINITIONS})
    add_definitions(-D_ENABLE_PETSC_)
  else()
  endif(PETSc_FOUND)
endif()

# From: https://github.com/Kitware/VTK/blob/master/CMake/FindNetCDF.cmake
option(ENABLE_NETCDF "Enable the output with the NetCDF library?" OFF)
if (ENABLE_NETCDF)
  set(NETCDF_F90 "YES")
  find_package(NetCDF REQUIRED)
  if(NETCDF_FOUND)
    target_link_libraries(${PROJECT_NAME} ${NETCDF_F90_LIBRARIES})
    target_include_directories(${PROJECT_NAME} PUBLIC ${NETCDF_F90_INCLUDE_DIR})
    add_definitions(-D_ENABLE_NETCDF_)
  endif(NETCDF_FOUND)
endif()
