################################################################################
# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

################################################################################
# Kind
################################################################################
set(KIND "0" CACHE STRING "Precision for real numbers")
set_property(CACHE KIND PROPERTY STRINGS "0;4;8;16")
add_definitions(-D_KIND_${KIND}_)
add_definitions(-D_KIND_=${KIND})
set(KIND ${KIND})
################################################################################

################################################################################
# Time
################################################################################
set(TIME_INTEGRATOR "FE" CACHE STRING "Choice for the time integrator")
set_property(CACHE TIME_INTEGRATOR PROPERTY STRINGS "FE;SSPRK11;SSPRK21;SSPRK31;SSPRK22;SSPRK32;SSPRK42;SSPRK52;SSPRK62;SSPRK72;SSPRK82;SSPRK33;SSPRK43;SSPRK53;SSPRK63;SSPRK73;SSPRK83;SSPRK54;SSPRK64;SSPRK74;SSPRK84")
add_definitions(-D_TIME_${TIME_INTEGRATOR}_)
add_definitions(-D_TIME_='${TIME_INTEGRATOR}')
set(TIME ${TIME_INTEGRATOR})
################################################################################


################################################################################
# Physics
################################################################################
set(PHY "ADV" CACHE STRING "Physics used")
set_property(CACHE PHY PROPERTY STRINGS "ADV;BURGERS;EULER;M1;P1;SW;TELE;HYDRORAD;M1MG")

if (${PHY} MATCHES "M1MG")
  set(PHY_M1MG_NB_GROUP "6" CACHE STRING "Number of groups in the M1 model")
  set_property(CACHE PHY_M1MG_NB_GROUP PROPERTY STRINGS "1;2;3;6")
  add_definitions(-D_PHY_M1MG_NB_GROUP=${PHY_M1MG_NB_GROUP})
  set(_PHY_OPT ${_PHY_OPT}_${PHY_M1MG_NB_GROUP})
endif()

if (NOT ${PHY} MATCHES "ADV" AND NOT ${PHY} MATCHES "BURGERS")
  option(ENABLE_SRC "Enable the source term?" OFF)
  if (ENABLE_SRC)
    add_definitions(-D_SRC_ENABLE_)
    set(_PHY_OPT ${_PHY_OPT}_Src)

    option(NO_AP "Disable the asymptotic preserving correction?" OFF)
    if (NO_AP)
      add_definitions(-D_SRC_DISABLE_AP_CORREC_)
      set(_PHY_OPT ${_PHY_OPT}_NoAP)
    endif()
  endif()
endif()

if (${PHY} MATCHES "EULER")
  set(PHY_EULER "IDEAL_GAS" CACHE STRING "Pressure law for the Euler model")
  set_property(CACHE PHY_EULER PROPERTY STRINGS "IDEAL_GAS;ISENTROPIC")
  add_definitions(-D_PHY_EULER_${PHY_EULER}_)
  if(${PHY_EULER} MATCHES "IDEAL_GAS")
  elseif(${PHY_EULER} MATCHES "ISENTROPIC")
    set(_PHY_OPT ${_PHY_OPT}_Is)
  endif()
endif()

add_definitions(-D_PHY_${PHY}_)
add_definitions(-D_PHY_='${PHY}${_PHY_OPT}')
################################################################################


################################################################################
# Fluxes
################################################################################
set(FLUX "RUS" CACHE STRING "Flux used")
set_property(CACHE FLUX PROPERTY STRINGS "RUS;HLL;HLLC;UPWIND;CENTERED;AT")

if (FLUX MATCHES "AT")
  set(FLUX_AT "EXEX" CACHE STRING "Full explicit or IMEX?")
  SET(TP_FLUX ON CACHE BOOL "Use the two-points flux?")
  set_property(CACHE FLUX_AT PROPERTY STRINGS "EXEX;IMEX")
  add_definitions(-D_FLUX_AT_${FLUX_AT}_)
  set(_FLUX_OPT ${_FLUX_OPT}_${FLUX_AT})
  if(FLUX_AT MATCHES "IMEX")
    set (ENABLE_PETSc ON CACHE BOOL "Enable PETSc library?")
  endif()
endif()

option(TP_FLUX "Use the two-points flux?" OFF)
if (TP_FLUX)
  add_definitions(-D_TP_FLUX_)
  set(_FLUX_OPT ${_FLUX_OPT}_TPFlux)
else()
  set(DLP_NB_PT_INTERP "3" CACHE STRING "Number of points for the DLP recontruction?")
  set_property(CACHE DLP_NB_PT_INTERP PROPERTY STRINGS "3;4")
  add_definitions(-D_DLP_NB_PT_INTERP_=${DLP_NB_PT_INTERP})
  set(_FLUX_OPT ${_FLUX_OPT}_DLP${DLP_NB_PT_INTERP})
endif()

option(TP_CORREC "Use the correction with the two-points flux?" OFF)
if (TP_CORREC)
  add_definitions(-D_TP_CORREC_)
  set(_FLUX_OPT ${_FLUX_OPT}_TPCorrec)
endif()

option(TP_CORREC_FULL "Use the correction with the two-points flux on all cells?" OFF)
if (TP_CORREC_FULL)
  add_definitions(-D_TP_CORREC_FULL_)
  set(_FLUX_OPT ${_FLUX_OPT}_TPCorrecFull)
endif()

if (ENABLE_SRC)
  option(FLUX_SRC_CENTER "Use a centered discretisation of the source term?" OFF)
  if (FLUX_SRC_CENTER)
    add_definitions(-D_FLUX_SRC_CENTER_)
    set(_FLUX_OPT ${_FLUX_OPT}_SrcCenter)
  endif()
endif()

add_definitions(-D_FLUX_${FLUX}_)
add_definitions(-D_FLUX_='${FLUX}${_FLUX_OPT}')
################################################################################

################################################################################
# High-order: MOOD and MUSCL
################################################################################
option(MOOD "Use the high-order version of the code using MOOD?" OFF)
if (MOOD)
  set(HO "MOOD")
  set(ENABLE_LAPACK ON)

  set(HO_MOOD_DEGREE "1" CACHE STRING "Degree of the polynomial recontruction")
  set_property(CACHE HO_MOOD_DEGREE PROPERTY STRINGS "0;1;2;3;4")
  set(HO ${HO}_P${HO_MOOD_DEGREE})

  option(MOOD_UNLIMITED "Use the unlimited version of MOOD?" OFF)
  if (MOOD_UNLIMITED)
    set(HO ${HO}_Ulimit)
    add_definitions(-D_HO_MOOD_UNLIMITED_)
  else()
    option(MOOD_PAD "Use the PAD criterion?" ON)
    if (MOOD_PAD)
      set(HO ${HO}_PAD)
      add_definitions(-D_HO_MOOD_PAD_)
    endif()

    option(MOOD_DMP "Use the DMP criterion?" OFF)
    if (MOOD_DMP)
      set(HO ${HO}_DMP)
      add_definitions(-D_HO_MOOD_DMP_)
    endif()

    if((MOOD_DMP OR MOOD_ED) AND HO_MOOD_DEGREE GREATER 1)
      option(MOOD_U2 "Use the U2 criterion?" OFF)
      if (MOOD_U2)
        set(HO ${HO}_u2)
        add_definitions(-D_HO_MOOD_U2_)
      endif()
    endif()

    option(MOOD_ED "Use the ED criterion?" OFF)
    if (MOOD_ED)
      set(HO ${HO}_ED)
      add_definitions(-D_HO_MOOD_ED_)
    endif()

    if(PHY MATCHES "EULER" OR PHY MATCHES "SW")
      option(MOOD_ENTROPIC "Use the entropic criterion?" OFF)
      if (MOOD_ENTROPIC)
        set(HO ${HO}_Ent)
        add_definitions(-D_HO_MOOD_ENTROPIC_)
      endif()
    endif()

    option(MOOD_CHECK_POLY_REC "Check the polynomial recontruction?" OFF)
    if (MOOD_CHECK_POLY_REC)
      set(HO ${HO}_CPR)
      add_definitions(-D_HO_MOOD_CHECK_POLY_REC_)
    endif()

    option(MOOD_FULL_DEC "Decrease all the cells degree to the minimal degree?" OFF)
    if (MOOD_FULL_DEC)
      set(HO ${HO}_Dec)
      add_definitions(-D_HO_MOOD_FULL_DEC_)
    endif()

    option(MOOD_MATRIX_SCALING "Rescale the matrix of MOOD?" ON)
    if (MOOD_MATRIX_SCALING)
      add_definitions(-D_HO_MOOD_MATRIX_SCALING_)
    endif()

  endif()

  add_definitions(-D_HO_=${HO})
  add_definitions(-D_HO_${HO}_)
  add_definitions(-D_HO_MOOD_)
  add_definitions(-D_HO_MOOD_MAX_DEGREE_=${HO_MOOD_DEGREE})
else()
  set(HO "NoHO")
endif()
################################################################################

################################################################################
# Mesh
################################################################################
set(MESH "P2D" CACHE STRING "The given mesh file is the diamond or the Primary mesh: D2P or P2D")
set_property(CACHE MESH PROPERTY STRINGS "D2P;P2D")

if(MESH MATCHES "P2D")
  option(MESH_RCM "Enable the RCM algorithm?" ON)
  if (MESH_RCM)
    add_definitions(-D_MESH_RCM_)
    set(_MESH_OPT ${_MESH_OPT}_RCM)
  endif()
  option(MESH_PERIODIC "Does the mesh contains periodic boundary conditions?" OFF)
  if (MESH_PERIODIC)
    add_definitions(-D_MESH_PERIODIC_)
    set(_MESH_OPT ${_MESH_OPT}_Per)
  endif()
endif()

add_definitions(-D_MESH_${MESH}_)
add_definitions(-D_MESH_='${MESH}${_MESH_OPT}')
set(MESH ${MESH})
################################################################################

################################################################################
# General info
################################################################################
set(INFO ${PROJECT_NAME}-${PHY}${_PHY_OPT}-${FLUX}${_FLUX_OPT}-${TIME}-${HO}-${MESH}${_MESH_OPT})
add_definitions(-D_INFO_='${INFO}')
################################################################################

################################################################################
# IO
################################################################################
add_definitions(-D_IO_OUTDIR_='out_${INFO}')

option(IO_ENABLE_DUMP "Enable dumps?" OFF)
if (IO_ENABLE_DUMP)
  add_definitions(-D_IO_ENABLE_DUMP_)
endif()

option(IO_ALL_DIGITS "Write all digits in the output files?" OFF)
if (IO_ALL_DIGITS)
  add_definitions(-D_IO_ALL_DIGITS_)
endif()

option(IO_ENABLE_VTK "Enable the writing in VTK files?" ON)
if (IO_ENABLE_VTK)
  add_definitions(-D_IO_ENABLE_VTK_)
endif()
################################################################################
