# From: https://majewsky.wordpress.com/2010/08/14/tip-of-the-day-cmake-and-doxygen/
# https://tty1.net/blog/2014/cmake-doxygen_en.html

option(BUILD_DOCUMENTATION "Use Doxygen to create the HTML based API documentation" OFF)
if(BUILD_DOCUMENTATION)
  find_package(Doxygen)
  if(NOT DOXYGEN_FOUND)
    message(FATAL_ERROR
      "Doxygen is needed to build the documentation. Please install it correctly")
  endif(NOT DOXYGEN_FOUND)

  # From: https://github.com/erisproject/voting/blob/master/cmake/Modules/FindMathJax.cmake
  # or: https://git.imaginary.ca/eris/creativity/blob/master/cmake/Modules/FindMathJax.cmake
  find_package(MathJax)
  if(MATHJAX_FOUND)
    set(MATHJAX_FOUND YES)
  else()
    set(MATHJAX_FOUND NO)
  endif(MATHJAX_FOUND)

  get_directory_property(COMPILE_DEFINITIONS COMPILE_DEFINITIONS)
  string (REPLACE ";" " " COMPILE_DEFINITIONS "${COMPILE_DEFINITIONS}")

  configure_file(${CMAKE_SOURCE_DIR}/doc/Doxyfile.in ${CMAKE_BINARY_DIR}/Doxyfile
    @ONLY)

  add_custom_target(doxydoc
    COMMAND ${DOXYGEN_EXECUTABLE} ${CMAKE_BINARY_DIR}/Doxyfile
    COMMAND ${CMAKE_COMMAND} -E echo "Documentation at file://${CMAKE_BINARY_DIR}/doxydoc/html/index.html"
    #"
    WORKING_DIRECTORY ${CMAKE_BINARY_DIR})
  add_custom_target(doxydoc_pdf
    COMMAND make
    DEPENDS doxydoc
    COMMAND ${CMAKE_COMMAND} -E echo "Documentation at file://${CMAKE_BINARY_DIR}/doxydoc/latex/refman.pdf"
    #"
    WORKING_DIRECTORY ${CMAKE_BINARY_DIR}/doxydoc/latex)

  set_directory_properties(PROPERTIES ADDITIONAL_MAKE_CLEAN_FILES ${CMAKE_BINARY_DIR}/doxydoc)
endif()
