################################################################################
# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

# Check if the compiler support Fortran90
if(NOT CMAKE_Fortran_COMPILER_SUPPORTS_F90)
  message(FATAL_ERROR "Fortran compiler does not support F90")
endif()

if (${CMAKE_SYSTEM_PROCESSOR} MATCHES "64")
  set(CMAKE_Fortran_FLAGS "${CMAKE_Fortran_FLAGS} \
  -m64")
else()
  set(CMAKE_Fortran_FLAGS "${CMAKE_Fortran_FLAGS} \
  -m32")
endif()

add_definitions(-D_FC_="${CMAKE_Fortran_COMPILER}")

if(${CMAKE_Fortran_COMPILER_ID} MATCHES "GNU")
  set(CMAKE_Fortran_FLAGS "${CMAKE_Fortran_FLAGS} \
  -cpp -ffree-line-length-none")
  set(CMAKE_Fortran_FLAGS_RELEASE "${CMAKE_Fortran_FLAGS_RELEASE} \
  -O3 -march=native")
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER "4.5.0")
    set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
     -flto")
  endif()
  set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
  -Wall -Wextra \
  -g -O0 \
  -Wimplicit-interface -fimplicit-none \
  -Waliasing -Wampersand \
  -Wintrinsics-std -Warray-temporaries \
  -Wcharacter-truncation -Wline-truncation \
  -Wtabs -Wuninitialized \
  -Wsurprising -fbacktrace \
  -ffpe-trap=zero,overflow,underflow,invalid")
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER "4.5.0")
    set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
    -fcheck=all -Wimplicit-procedure")
  endif()
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER "4.6.0")
    set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
    -Wconversion-extra")
  endif()
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER "4.7.0")
    set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
    -Wreal-q-constant")
  endif()
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER "4.8.0")
    set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
    -Wrealloc-lhs -Wrealloc-lhs-all")
  endif()
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER "5.1.0")
    set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
    -Wuse-without-only -Wc-binding-type")
  endif()
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER "6.1.0")
    set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
    -Winteger-division")
  endif()
  set(CMAKE_Fortran_FLAGS_PROFILING "-pg -fbacktrace -g")
  set(CMAKE_Fortran_FLAGS_COVERAGE "--coverage -fbacktrace -g")

  add_definitions(-D_FC_GNU_)

elseif(${CMAKE_Fortran_COMPILER_ID} MATCHES "Intel")
  set(CMAKE_Fortran_FLAGS "${CMAKE_Fortran_FLAGS} \
  -fpp")
  set(CMAKE_Fortran_FLAGS_RELEASE "${CMAKE_Fortran_FLAGS_RELEASE} \
  -xHost \
  -mp1 -fno-fnalias -unroll-aggressive \
  -O3 \
  -ipo \
  ")
  set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
  -warn all -check all -debug all \
  -traceback -g -O0 \
  -fp-stack-check \
  -fpe0 -ftrapuv")
  set(CMAKE_Fortran_FLAGS_PROFILING "-p -traceback -g")
  set(CMAKE_Fortran_FLAGS_COVERAGE "-prof-gen=srcpos -traceback -g")

  add_definitions(-D_FC_INTEL_)

elseif(${CMAKE_Fortran_COMPILER_ID} MATCHES "PGI")
  set(CMAKE_Fortran_FLAGS "${CMAKE_Fortran_FLAGS} \
  -Mpreprocess")
  set(CMAKE_Fortran_FLAGS_RELEASE "${CMAKE_Fortran_FLAGS_RELEASE} \
  -O3 -Mipa=safeall,fast,inline -fastsse")
  set(CMAKE_Fortran_FLAGS_DEBUG "${CMAKE_Fortran_FLAGS_DEBUG} \
  -C -g -traceback \
  -Ktrap=denorm,divz,fp,inv,ovf,unf")
  set(CMAKE_Fortran_FLAGS_PROFILING "-Mprof -traceback -g -pg")

  add_definitions(-D_FC_PGI_)

endif()

option(ENABLE_OMP "Enable OpenMP support" OFF)
if(ENABLE_OMP)
  find_package(OpenMP REQUIRED)
  add_definitions(-D_FC_PAR_OMP_)
  set(CMAKE_Fortran_FLAGS "${CMAKE_Fortran_FLAGS} ${OpenMP_Fortran_FLAGS}")
  #"
endif()

option(ENABLE_MPI "Enable MPI support" OFF)
if(ENABLE_MPI)
  find_package(MPI REQUIRED)
  add_definitions(-D_FC_PAR_MPI_)
  target_include_directories(${PROJECT_NAME} PUBLIC ${MPI_Fortran_INCLUDE_PATH})
  target_link_libraries(${PROJECT_NAME} ${MPI_Fortran_LIBRARIES})
  set(CMAKE_Fortran_FLAGS "${CMAKE_Fortran_FLAGS} ${MPI_Fortran_FLAGS}")
  #"
endif()


if(CMAKE_BUILD_TYPE MATCHES "Debug")
  add_definitions(-D_FFLAGS_="${CMAKE_Fortran_FLAGS} ${CMAKE_Fortran_FLAGS_DEBUG}")
elseif(CMAKE_BUILD_TYPE MATCHES "Release")
  add_definitions(-D_FFLAGS_="${CMAKE_Fortran_FLAGS} ${CMAKE_Fortran_FLAGS_RELEASE}")
endif()
