```
    _       _              _    ____ ____  ____
   / \   __| |_ __ ___    / \  |  _ \___ \|  _ \
  / _ \ / _` | '_ ` _ \  / _ \ | |_) |__) | | | |
 / ___ \ (_| | | | | | |/ ___ \|  __// __/| |_| |
/_/   \_\__,_|_| |_| |_/_/   \_\_|  |_____|____/
```
> ASCII art done with [FIGlet](http://www.figlet.org/)

[![pipeline status](https://plmlab.math.cnrs.fr/florian.blachere/AdmAP2D/badges/master/pipeline.svg)](https://plmlab.math.cnrs.fr/florian.blachere/AdmAP2D/pipelines)
[![coverage report](https://plmlab.math.cnrs.fr/florian.blachere/AdmAP2D/badges/master/coverage.svg)](https://florian.blachere.pages.math.cnrs.fr/AdmAP2D/coverage/)

# AdmAP2D
An admissibility and asymptotic-preserving finite volume scheme for systems of
conservation laws with source terms on 2D unstructured meshes

# Usage

## Compilation
### Make
Just type `make` in the top directory, see [`mk/scheme.mk`](mk/scheme.mk) for the
different options related to the scheme and [`mk/libs.mk`](mk/libs.mk) for the
libraries that can be used.

The different compilation options for `gfortran` and `ifort` are located in
[`mk/gnu_f.mk`](mk/gnu_f.mk) and [`mk/intel_f.mk`](mk/intel_f.mk)

### CMake
It is also possible to use [CMake](https://www.cmake.org/) to configure the code, then build it
with `make`:
```
$ mkdir -p build
$ cd build
$ cmake .. # or 'ccmake ..' to get access to the options
$ make
```

### Meson
Finally [Meson](http://mesonbuild.com/) can be used to configure the code, then build it
with [Ninja](https://ninja-build.org/):
```
$ meson build
$ cd build
$ ninja
```
The available options can be displayed with `mesonconf` in the build directory or in the
[`meson_options.txt`](meson_options.txt) file.

## Run
`$DESTDIR/$BUILDDIR/bin/AdmAP2D --mesh path/to/mesh/file.msh --test_case TestCase --restart path/to/restart/file`,
where:
 * `$BUILDDIR` is something like `build_AdmAP2D-$PHY-$FLUX-$TIME-$ORDER-$MESH-$ARCH-$COMPILER`,
 * `$PHY` is the physics,
 * `$FLUX` is the flux and the reconstruction used,
 * `$TIME` is the time scheme used,
 * `$ORDER` describes the high order method used,
 * `$MESH` is the mesh used,
 * `$ARCH` is the architecture where the code was build,
 * `$COMPILER` is the compiler and the options used.

This will run the code and the output will be writen
in `$DESTDIR/$BUILDDIR/out/$PHY-$FLUX-$MeshName-$TestCase`
as VTK files (for [VisIt](https://wci.llnl.gov/simulation/computer-codes/visit)
or [ParaView](https://www.paraview.org/)).

Some sample meshes generated by [Gmsh](http://gmsh.info/) are available in the [`mesh`](mesh) folder.

## Scripts
Some scripts are available in the [`scripts`](scripts) directory to
simplify the [compilation](scripts/compile.sh) and the [run](scripts/run.sh).

They can be used as skeletons to launch others test cases, for the list of the possible test cases
one as to look to the various physic modules.

## Testing
To test if the code run well, it is possible to test it with a constant test
case and appropriate boundaries: nothing has to move.

To test the compilation, it is possible to use the script
[`tests_compile.sh`](scripts/tests_compile.sh) in order to check all the compilations (using
different combinations of parameters).

## Modifications
In order to add or change the property of the physics used see the files
[`src/physics/physics.f90`](src/physics/physics.f90) and
[`src/physics/physics_var.f90`](src/physics/physics_var.f90) that are used as wrappers for the
routines and the variables. The module for each physics are `physics_PHY.f90` and
`physics_PHY_var.f90` in the folder [`src/physics`](src/physics).  There is also the files in
[`src/asymptotic`](src/asymptotic) for everything linked to the source term (for instance the
computations of the asymptotic correction).

The tags for the boundary conditions can be changed in
[`src/mesh/type_mesh.f90`](src/mesh/type_mesh.f90) in order to fit with the tags in the `.msh`
files.

Numerical fluxes can be added in the [`src/flux/flux_1D.f90`](src/flux/flux_1D.f90) file with the proper
preprocessing variable in [`mk/scheme.mk`](mk/scheme.mk).

# Documentation
## Doxygen
[Doxygen documentation](https://florian.blachere.pages.math.cnrs.fr/AdmAP2D/doc), generated with
`doxygen` or `make doxydoc` or `cmake -DBUILD_DOCUMENTATION=ON && make doxydoc`

## Articles
Mainly [AAP2D](https://dx.doi.org/10.1016/j.jcp.2016.03.045) (or on
[HAL](https://hal.archives-ouvertes.fr/hal-01293971)) for the first first-order scheme
and [AAP2D-HO](https://dx.doi.org/10.1016/j.cma.2017.01.012) (or on
[HAL](https://hal.archives-ouvertes.fr/hal-01436735)) for the high-order scheme.
You may also look at references therein and bibliography used in the code.

The scheme is built using the [Droniou and Le Potier
scheme](https://dx.doi.org/10.1137/090770849) (or on [HAL]
(https://hal.archives-ouvertes.fr/hal-00808694)) and the asymptotic-preserving
technique developed by [Berthon and Turpault](https://dx.doi.org/10.1002/num.20586).

## PhD thesis:
* [F. Blachère](https://tel.archives-ouvertes.fr/tel-01373877):
 HLL-DLP-AP scheme,
* [C. Céline Sarazin-Desbois](https://hal.archives-ouvertes.fr/tel-00814182):
 physics for the M1 model and first idea for the 2D unstructured scheme,
* [M. Girardin](https://hal.archives-ouvertes.fr/tel-01127428):
 acoustic-transport splitting with IMEX,
* [R. Turpault](https://hal.archives-ouvertes.fr/tel-00004620):
 multi-group M1 model.

# Requirements
* [makedepf90](http://personal.inet.fi/private/erikedelmann/makedepf90/) is optional to generate
  the dependency list and build the project (see [`mk/deps.mk`](mk/deps.mk)), it's also possible to use
  [CMake](https://cmake.org/) or [Meson](http://mesonbuild.com/) to configure the code (see
  [`compile.sh`](scripts/compile.sh)),
* [Doxygen](https://www.stack.nl/~dimitri/doxygen/)
  is needed to generate the documentation from the source code,
* a recent Fortran90 compiler (tested with [`gfortran`](https://gcc.gnu.org/fortran/),
  [`ifort`](https://software.intel.com/en-us/fortran-compilers)
  and [`pgfortran`](https://www.pgroup.com/products/community.htm)),
* a library that provides BLAS and LAPACK routines (for instance:
  [LAPACK](http://www.netlib.org/lapack/),
  [OpenBLAS](http://www.openblas.net/), [ATLAS](http://www.netlib.org/atlas/),
  [MKL](https://software.intel.com/en-us/intel-mkl/), ...), this is especially
  needed for the high-order version of the code,
* the parallel version needs an [OpenMP](http://openmp.org/wp/) implementation
  (tested with the [GNU](https://gcc.gnu.org/projects/gomp/) and
  [Intel](https://www.openmprtl.org/) implementations),
* small tools like `hostname`, `uname`, `basename`, `grep`, `tput`,
  `readlink`, `find`, `echo` are needed,
* code coverage needs [Gcov](https://gcc.gnu.org/onlinedocs/gcc/Gcov.html) (and
  [LCOV](http://ltp.sourceforge.net/coverage/lcov.php) or [Gcovr](http://gcovr.com/)) or the
  tools from the Intel suite with the helps of [`cov_gnu.sh`](scripts/cov_gnu.sh) and
  [`cov_intel.sh`](scripts/cov_intel.sh),
* profiling can be done using [gprof](https://sourceware.org/binutils/docs/gprof/) or
  [Scalasca](http://www.scalasca.org/) or [ompP](http://www.ompp-tool.com/), with the scripts [`gprof.sh`](scripts/gprof.sh) and
  [`scalasca.sh`](scripts/scalasca.sh),
* [Pandoc](http://pandoc.org/) is needed to convert the markdown files to somethiong more
  meaningfull with the `md2ZZZ` rules in the [Makefile](Makefile).

# Authors
* [Chrisophe Berthon](https://www.math.sciences.univ-nantes.fr/~berthon/): <Christophe.Berthon@univ-nantes.fr>,
  [Université de Nantes][UN], [Laboratoire de Mathématiques Jean Leray][LMJL],
* [Florian Blachère](https://blacheref.perso.math.cnrs.fr/): <florian.blachere@utt.fr>,
  [Université de technologie de Troyes][UTT], [Institut Charles Delaunay][ICD], [GAMMA3][GAMMA3]
* [Sébastien Guisset](https://guisset.perso.math.cnrs.fr/): <sebguisset@gmail.com>,
  [Université de Versailles Saint-Quentin-en-Yvelines][UVSQ], [Laboratoire de Mathématiques de Versailles][LMV],
* [Guy Moebs](https://www.math.sciences.univ-nantes.fr/~moebs/): <guy.moebs@univ-nantes.fr>,
  [CNRS][CNRS], [Laboratoire de Mathématiques Jean Leray][LMJL],
* Céline Sarazin-Desbois:
  [Université de Nantes][UN], [Laboratoire de Mathématiques Jean Leray][LMJL],
* Maxime Stauffert: <maxime.stauffert@gmail.com>,
  [Université de Versailles Saint-Quentin-en-Yvelines][UVSQ], [Laboratoire de Mathématiques de Versailles][LMV],
* [Rodolphe Turpault](https://www.math.u-bordeaux.fr/~rturpault/): <rodolphe.turpault@u-bordeaux.fr>,
  [Bordeaux-INP][BINP], [Institut de Mathématiques de Bordeaux][IMB].

# License
GPLv3, see the [LICENSE.md](LICENSE.md) file.

* Copyright © 2017-2018 Université de technologie de Troyes,
* Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
* Copyright © 2013-2018 CNRS,
* Copyright © 2013-2016 Université de Nantes.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


[BINP]: https://www.bordeaux-inp.fr/
[CNRS]: http://www.cnrs.fr/
[GAMMA3]: http://gamma3.utt.fr/
[ICD]: http://icd.utt.fr/
[IMB]: http://www.math.u-bordeaux1.fr/imb/
[LMJL]: http://www.math.sciences.univ-nantes.fr
[LMV]: http://lmv.math.cnrs.fr/
[UN]: http://www.univ-nantes.fr/
[UTT]: http://www.utt.fr/
[UVSQ]: http://www.uvsq.fr/
