# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

VERSION=gnu
GCC_VER_MAJOR=$(shell gcc -dumpversion 2> /dev/null | cut -d. -f 1)
GCC_VER_MINOR=$(shell gcc -dumpversion 2> /dev/null | cut -d. -f 2)
GCC_VER_PATCH=$(shell gcc -dumpversion 2> /dev/null | cut -d. -f 3)

MODCOMMAND=-J$(MODDIR)
FFLAGS+=-cpp -ffree-line-length-none
#FFLAGS+=-fexternal-blas
ifeq ($(shell [ $(GCC_VER_MAJOR) -ge 5 ] && echo 1), 1)
	FFLAGS+=-fdiagnostics-color=always
endif

ifeq ($(OMP),1)
	PREPROC+=-D_FC_PAR_OMP_
	FFLAGS+=-fopenmp
	LDFLAGS+=-lgomp
	VERSION:=$(VERSION)_omp
endif # OMP

ifeq ($(MPI),1)
	PREPROC+=-D_FC_PAR_MPI_
	VERSION:=$(VERSION)_mpi
endif # MPI

ifeq ($(PROFILING), 1)
	FFLAGS+=-pg -fbacktrace -g
	PREPROC+=-D_PROFILING_
	LDFLAGS+=-pg
	VERSION:=$(VERSION)_prof
endif # PROFILING

ifeq ($(COVERAGE), 1)
	FFLAGS+=-fbacktrace -g
	FFLAGS+=-fprofile-arcs -ftest-coverage
	PREPROC+=-D_COVERAGE_
	LDFLAGS+=-lgcov
	VERSION:=$(VERSION)_cov
endif # COVERAGE

ifeq ($(DEBUG), 1)
	PREPROC+=-D_DEBUG_
	FFLAGS+=-Wall -Wextra
	FFLAGS+=-g -O0
	FFLAGS+=-Wimplicit-interface -Wintrinsics-std
	FFLAGS+=-Warray-temporaries
	FFLAGS+=-Wcharacter-truncation -Wline-truncation
	FFLAGS+=-Wtabs -Wuninitialized -Wsurprising
ifeq ($(shell [ $(GCC_VER_MAJOR) -ge 4 ] && echo 1), 1)
ifeq ($(shell [ $(GCC_VER_MINOR) -ge 5 ] && echo 1), 1)
		FFLAGS+=-fcheck=all -Wimplicit-procedure
endif
ifeq ($(shell [ $(GCC_VER_MINOR) -ge 6 ] && echo 1), 1)
		FFLAGS+=-Wconversion-extra
endif
ifeq ($(shell [ $(GCC_VER_MINOR) -ge 7 ] && echo 1), 1)
		FFLAGS+=-Wreal-q-constant
endif
ifeq ($(shell [ $(GCC_VER_MINOR) -ge 8 ] && echo 1), 1)
		FFLAGS+=-Wrealloc-lhs -Wrealloc-lhs-all
endif
endif
ifeq ($(shell [ $(GCC_VER_MAJOR) -ge 5 ] && echo 1), 1)
		FFLAGS+=-Wuse-without-only -Wc-binding-type
endif
ifeq ($(shell [ $(GCC_VER_MAJOR) -ge 6 ] && echo 1), 1)
		FFLAGS+=-Winteger-division
endif
	FFLAGS+=-fbacktrace
	FFLAGS+=-ffpe-trap=zero,overflow,underflow,invalid
	#FFLAGS+=-std=f2008 -fall-intrinsics
	#FFLAGS+=-finit-local-zero
	#FFLAGS+=-finit-real=snan
	FFLAGS+=-fimplicit-none
	VERSION:=$(VERSION)_g
else
	#FFLAGS+=-O0 -march=native
	#FFLAGS+=-O1 -march=native
	#FFLAGS+=-O2 -march=native
	FFLAGS+=-O3 -march=native
endif # DEBUG

ifeq ($(LINK_OPTIM),1)
ifeq ($(shell [ $(GCC_VER_MAJOR) -ge 4 ] && echo 1), 1)
ifeq ($(shell [ $(GCC_VER_MINOR) -ge 5 ] && echo 1), 1)
			FFLAGS+=-flto
			LDFLAGS+=-flto
			VERSION:=$(VERSION)_lto
endif
endif
endif # INLINING

PREPROC+=-D_FC_GNU_
