# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

VERSION=intel
IFORT_VER_MAJOR=$(shell ifort --version 2> /dev/null | head -n 1 | cut -d ' ' -f 3 | cut -d '.' -f 1)

MODCOMMAND=-module $(MODDIR)
FFLAGS+=-fpp

ifeq ($(OMP),1)
	PREPROC+=-D_FC_PAR_OMP_
ifeq ($(shell [ $(IFORT_VER_MAJOR) -ge 15 ] && echo 1), 1)
		FFLAGS+=-qopenmp
else
		FFLAGS+=-openmp
endif
	LDFLAGS+=-liomp5
	VERSION:=$(VERSION)_omp
endif # OMP

ifeq ($(MPI),1)
	PREPROC+=-D_FC_PAR_MPI_
	VERSION:=$(VERSION)_mpi
endif # MPI

ifeq ($(PROFILING), 1)
	FFLAGS+=-p -traceback -g
	PREPROC+=-D_PROFILING_
	LDFLAGS+=-p
	VERSION:=$(VERSION)_prof
endif # PROFILING

ifeq ($(COVERAGE), 1)
	FFLAGS+=-traceback -g
	FFLAGS+=-prof-gen=srcpos
	PREPROC+=-D_COVERAGE_
	VERSION:=$(VERSION)_cov
endif # COVERAGE

ifeq ($(DEBUG), 1)
	PREPROC+=-D_DEBUG_
	FFLAGS+=-warn all -check all -debug all
	FFLAGS+=-traceback -g -O0
	FFLAGS+=-fp-stack-check
	FFLAGS+=-fpe0 -ftrapuv
	#FFLAGS+=-fno-inline
	#FFLAGS+=-Winline
	#FFLAGS+=-check noarg_temp_created
	#FFLAGS+=-warn nounused
	VERSION:=$(VERSION)_g
else
	FFLAGS+=-xHost -g -traceback
	#FFLAGS+=-O0
	#FFLAGS+=-O1
	FFLAGS+=-O2
	#FFLAGS+=-O3
	#FFLAGS+=-fast
	FFLAGS+=-mp1 -fno-fnalias -unroll-aggressive
endif # DEBUG

ifneq ($(COVERAGE), 1)
ifeq ($(LINK_OPTIM),1)
		FFLAGS+=-ipo
		LDFLAGS+=-ipo
		VERSION:=$(VERSION)_ipo
endif # LINK_OPTIM
endif # COVERAGE

PREPROC+=-D_FC_INTEL_
