#!/bin/bash
# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Script to compile the code with the different options

#######################################################################
## Code configuration
#######################################################################
# Name of the binary
CODE=AdmAP2D
# Physics used: ADV, BURGERS, EULER, HYDRORAD, HYPHEAT, M1, M1MG, P1, SW, TELE
PHY=EULER
_PHY_OPT=_Src
# Time integrators: FE, SSPRK22, SSPRK33, SSPRK54, ...
TIME=SSPRK22
#
ORDER=MOOD_P1_PAD
# Numerical flux: RUS, HLL, HLLC
FLUX=HLL
DLP_NB_PT_INTERP=3
_FLUX_OPT=_DLP${DLP_NB_PT_INTERP}_TPCorrec
# Way to create the mesh: Primary to Diamond (P2D) or Diamond to Primary (D2P)
MESH=P2D
_MESH_OPT=_RCM
#
ARCH=$(uname -m)
# Fortran compiler
FC=gnu
_FC_OPT=_omp
# Root directory
ROOTDIR=$(pwd)
# Destination directory
#DESTDIR=~/work/runs/${CODE}/${PHY}${_PHY_OPT}-${ORDER}/${DESTDIR_OPT}
DESTDIR=./runs
DESTDIR=$(readlink -m "${DESTDIR}")
BUILDDIR=build_${CODE}-${PHY}${_PHY_OPT}-${FLUX}${_FLUX_OPT}-${TIME}-${ORDER}-${MESH}${_MESH_OPT}-${ARCH}-${FC}${_FC_OPT}
BIN=${BUILDDIR}/bin/${CODE}
#
MAKEFLGAS=-j4
#######################################################################


#######################################################################
## Compile
#######################################################################
if hash cmake 2>/dev/null; then
  mkdir -p -- "${DESTDIR}/${BUILDDIR}"
  cd "${DESTDIR}/${BUILDDIR}" || exit 1
  cmake "${ROOTDIR}" -DCMAKE_BUILD_TYPE:STRING=Release \
    -DPHY:STRING=${PHY} -DFLUX:STRING=${FLUX} \
    -DPHY_EULER:STRING="ISENTROPIC" -DPHY_M1MG_NBGROUP:SRING=6 \
    -DMOOD:BOOL=ON -DMOOD_MAX_DEGREE:STRING=1 \
    -DMOOD_PAD:BOOL=ON -DMOOD_DMP:BOOL=OFF -DMOOD_ED:BOOL=OFF \
    -DMOOD_U2:BOOL=OFF -DMOOD_ENTROPIC:BOOL=OFF -DMOOD_CHECK_POLY_REC:BOOL=OFF \
    -DMOOD_UNLIMITED:BOOL=OFF -DMOOD_FULL_DEC:BOOL=OFF \
    -DTIME:STRING=${TIME} \
    -DTP_CORREC:BOOL=ON -DTP_FLUX:BOOL=OFF -DTP_CORREC_FULL:BOOL=OFF \
    -DENABLE_SRC:BOOL=ON -DFLUX_SRC_CENTER:BOOL=OFF -DNO_AP:BOOL=OFF \
    -DMESH=${MESH} -DMESH_RCM:BOOL=ON -DMESH_PERIODIC:BOOL=OFF \
    -DDLP_NB_PT_INTERP:STRING=${DLP_NB_PT_INTERP} \
    -DENABLE_OMP:BOOL=ON -DENABLE_LAPACK:BOOL=ON \
    -DIO_ENABLE_DUMP:BOOL=ON -DENABLE_HDF5:BOOL=OFF -DENABLE_NC:BOOL=OFF \
    -DIO_ENABLE_VTK:BOOL=ON
  make ${MAKEFLGAS}

else
  make ${MAKEFLGAS} \
    PHY=${PHY} FLUX=${FLUX} \
    PHY_EULER_ISENTROPIC=0 M1MG_NB_GROUP=6 \
    MOOD=1  \
    MOOD_UNLIMITED=0 MOOD_PAD=1 MOOD_DMP=0 MOOD_ED=0 MOOD_U2=0 MOOD_ENTROPIC=0 \
    MOOD_FULL_DEC=0 MOOD_CHECK_POLY_REC=0 \
    TIME=${TIME} \
    TP_CORREC=1 TP_FLUX=0 TP_CORREC_FULL=0 \
    SRC=1 FLUX_SRC_CENTER=0 NO_AP=0 \
    MESH=${MESH} RCM=1 PERIODIC=0 \
    DLP_NB_PT_INTERP=${DLP_NB_PT_INTERP} \
    OMP=1 DEBUG=0 LINK_OPTIM=0 \
    OMPP=0 SCALASCA=0 PROFILING=0 COVERAGE=0 \
    DUMP=1 HDF5=0 NC=0 VTK=1 \
    DESTDIR="${DESTDIR}"
fi

#######################################################################

exit 0
