#!/bin/bash
# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Run

# Fusion stdout and stderr
#PBS -j eo

# Time [hh:mm:ss]
#PBS -l walltime=119:00:00

# Resources
#PBS -l select=1:ncpus=12:mem=20Gb

# Job name
#PBS -N SodP1_KL

# Notifications [abort|begin|end]
#PBS -m abe

# Mail
#PBS -M florian.blachere@uvsq.fr
#######################################################################
## Code configuration
#######################################################################
# Name of the binary
CODE=AdmAP2D
# Physics used: ADV, BURGERS, EULER, HYDRORAD, HYPHEAT, M1, M1MG, P1, SW, TELE
PHY=EULER
_PHY_OPT=_Src
# Time integrators: FE, SSPRK22, SSPRK33, SSPRK54, ...
TIME=SSPRK22
#
ORDER=MOOD_P1_PAD
# Numerical flux: RUS, HLL, HLLC
FLUX=HLL
DLP_NB_PT_INTERP=3
_FLUX_OPT=_DLP${DLP_NB_PT_INTERP}_TPCorrec
# Way to create the mesh: Primary to Diamond (P2D) or Diamond to Primary (D2P)
MESH=P2D
_MESH_OPT=_RCM
#
ARCH=$(uname -m)
# Fortran compiler
FC=gnu
_FC_OPT=_omp
# Destination directory
#DESTDIR=~/work/runs/${CODE}/${PHY}${_PHY_OPT}-${ORDER}/${DESTDIR_OPT}
DESTDIR=./runs
DESTDIR=$(readlink -m "${DESTDIR}")
# Binary
BIN=build_${CODE}-${PHY}${_PHY_OPT}-${FLUX}${_FLUX_OPT}-${TIME}-${ORDER}-${MESH}${_MESH_OPT}-${ARCH}-${FC}${_FC_OPT}/bin/${CODE}
#######################################################################

#######################################################################
## Test case and meshes
#######################################################################
test_case=StepM3
#MESHDIR=~/work/mesh/tri/step
MESHDIR=$(readlink -m "mesh")
mesh=step_tri
#tag_meshes=$(seq -f %02g 1 6)
tag_meshes='01'
#######################################################################

#######################################################################
## Runtime parameters
#######################################################################
export OMP_DISPLAY_ENV=TRUE
export OMP_DYNAMIC=FALSE
#export OMP_NUM_THREADS=12
#threads=$(seq -f %02g 1 12)
threads=04
if [ ${FC} = 'gnu' ]; then
  export OMP_PLACES=CORES # THREADS, CORES, SOCKETS
  export OMP_PROC_BIND=CLOSE # TRUE, FALSE, MASTER, CLOSE, SPREAD
fi
export OMP_SCHEDULE=STATIC # STATIC, DYNAMIC, GUIDED, AUTO
export OMP_STACKSIZE=512M
export MKL_NUM_THREADS=1
if [ ${FC} = 'intel' ]; then
  # verbose/noverbose, granularity=core/thread/fine, scatter,compact
  export KMP_AFFINITY=verbose
fi
ulimit -s unlimited
ulimit -v unlimited
#######################################################################

#######################################################################
## Run
#######################################################################
cd "${DESTDIR}" || (echo "${DESTDIR} does not exist" && exit 1)

if [ -s "${BIN}" ]
then
  for tag_mesh in ${tag_meshes}
  do
    for thread in ${threads}
    do
      export OMP_NUM_THREADS=${thread}

      logfile=log_${CODE}-${PHY}${_PHY_OPT}-${FLUX}${_FLUX_OPT}-${TIME}-${ORDER}-DLP${DLP_NB_PT_INTERP}-${MESH}${_MESH_OPT}-${ARCH}-${FC}${_FC_OPT}-${mesh}_${tag_mesh}-${test_case}-T${thread}

      if [ -z "${restart_file}"] ; then
        /usr/bin/time -v  -- \
        "./${BIN}" --mesh="${MESHDIR}/${mesh}_${tag_mesh}.msh" \
        --test_case=${test_case} \
        >> "${logfile}" 2>&1
      else
        /usr/bin/time -v  -- \
        "./${BIN}" --mesh="${MESHDIR}/${mesh}_${tag_mesh}.msh" \
        --test_case=${test_case} \
        --restart=${restart_file} \
        >> "${logfile}" 2>&1
      fi

      echo "${mesh}_${tag_mesh} done, logfile: ${DESTDIR}/${logfile}"
    done
  done
else
  echo "${DESTDIR}/${BIN} does not exist"
  exit 1
fi
#######################################################################
exit 0
