#!/bin/bash
# Copyright © 2017-2018 Université de technologie de Troyes,
# Copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
# Copyright © 2013-2018 CNRS,
# Copyright © 2013-2016 Université de Nantes.
# Author: Florian Blachère
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Compile everything and see if there is trouble

#Color and bold
red="\e[31m\e[1m"
reset="\e[0m"

NbTests=1

if [[ ${FULL_TEST} -eq 1 ]]
then
  PHYs=('ADV' 'BURGERS' 'EULER' 'HYDRORAD' 'HYPHEAT' 'M1' 'M1MG' 'P1' 'SW' 'TELE')
  FLUXs=('RUS' 'HLL' 'HLLC')
  TIMEs=('FE' 'SSPRK22' 'SSPRK33' 'RK3TVD' 'SSPRK54')
  MESHs=('D2P' 'P2D')
  RCMs=('0' '1')
  OMPs=('0' '1')
  DUMPs=('0' '1')
  VTKs=('0' '1')
  SRCs=('0' '1')
  NO_APs=('0' '1')
  TP_FLUXs=('0' '1')
  TP_CORRECs=('0' '1')
  TP_CORREC_FULLs=('0' '1')
  MOODs=('-1' '0' '1' '2' '3' '4' '5')
  MOOD_UNLIMITEDs=('0' '1')
  MOOD_PADs=('0' '1')
  MOOD_DMPs=('0' '1')
  MOOD_EDs=('0' '1')
  MOOD_U2s=('0' '1')
  MOOD_ENTROPICs=('0' '1')
else
  PHYs=('HYDRORAD')
  FLUXs=('HLL')
  TIMEs=('SSPRK54')
  MESHs=('P2D')
  RCMs=('1')
  OMPs=('1')
  DUMPs=('1')
  VTKs=('1')
  SRCs=('0' '1')
  NO_APs=('0')
  TP_FLUXs=('0')
  TP_CORRECs=('1')
  TP_CORREC_FULLs=('0')
  MOODs=('5')
  MOOD_UNLIMITEDs=('0')
  MOOD_PADs=('1')
  MOOD_DMPs=('1')
  MOOD_EDs=('0')
  MOOD_U2s=('1')
  MOOD_ENTROPICs=('1')
fi

TotTests="$((${#PHYs[@]} * ${#FLUXs[@]} * ${#TIMEs[@]} * ${#MESHs[@]} * ${#RCMs[@]} * ${#OMPs[@]} * ${#DUMPs[@]} * ${#VTKs[@]} * ${#SRCs[@]} * ${#NO_APs[@]} * ${#TP_FLUXs[@]} * ${#TP_CORRECs[@]} * ${#TP_CORREC_FULLs[@]} * ${#MOODs[@]} * ${#MOOD_UNLIMITEDs[@]} * ${#MOOD_PADs[@]} * ${#MOOD_DMPs[@]} * ${#MOOD_EDs[@]} * ${#MOOD_U2s[@]} * ${#MOOD_ENTROPICs[@]}))"

echo "Running ${TotTests} tests"
echo "======================================"

ROOTDIR=$(pwd)
TESTDIR=$(readlink -m ./tests)
BUILDDIR=build
MAKEFLAGS=-j$(nproc)

rm -rf "${TESTDIR}"
mkdir -p "${TESTDIR}"

for PHY in "${PHYs[@]}"; do
  for FLUX in "${FLUXs[@]}" ; do
    for TIME in "${TIMEs[@]}" ; do
      for MESH in "${MESHs[@]}" ; do
        for RCM in "${RCMs[@]}" ; do
          for OMP in "${OMPs[@]}" ; do
            for DUMP in "${DUMPs[@]}" ; do
              for VTK in "${VTKs[@]}"; do
                for SRC in "${SRCs[@]}" ; do
                  for NO_AP in "${NO_APs[@]}" ; do
                    for TP_FLUX in "${TP_FLUXs[@]}" ; do
                      for TP_CORREC_FULL in "${TP_CORREC_FULLs[@]}" ; do
                        for TP_CORREC in "${TP_CORRECs[@]}" ; do
                          for MOOD in "${MOODs[@]}" ; do
                            for MOOD_UNLIMITED in "${MOOD_UNLIMITEDs[@]}" ; do
                              for MOOD_PAD in "${MOOD_PADs[@]}" ; do
                                for MOOD_DMP in "${MOOD_DMPs[@]}" ; do
                                  for MOOD_ED in "${MOOD_EDs[@]}" ; do
                                    for MOOD_U2 in "${MOOD_U2s[@]}" ; do
                                      for MOOD_ENTROPIC in "${MOOD_ENTROPICs[@]}" ; do

                                        logfile="${TESTDIR}/${PHY}-${FLUX}-${TIME}-${MESH}_RCM${RCM}-OMP${OMP}-DUMP${DUMP}-VTK${VTK}-Src${SRC}_NoAP${NO_AP}-TPFlux${TP_FLUX}_TPCorrec${TP_CORREC}_TPCorrecFull${TP_CORREC_FULL}-MOOD${MOOD}_Ulimi${MOOD_UNLIMITED}_PAD${MOOD_PAD}_DMP${MOOD_DMP}_ED${MOOD_ED}_u2${MOOD_U2}_Ent${MOOD_ENTROPIC}.log"
                                        echo -e "Compiling ${logfile} ..."
                                        echo -e "Test ${NbTests}/${TotTests}"

                                        if [[ ! -e /.dockerinit ]]
                                        then
                                          # Continue on failure and print errors in logs
                                          if hash cmake 2>/dev/null
                                          then
                                            mkdir -p -- "${TESTDIR}/${BUILDDIR}"
                                            cd "${TESTDIR}/${BUILDDIR}" || exit 1
                                            cmake "${ROOTDIR}" -DCMAKE_BUILD_TYPE:STRING=Debug \
                                              -DPHY:STRING="${PHY}" -DFLUX:STRING="${FLUX}" \
                                              -DPHY_EULER:STRING="IDEAL_GAS" -DPHY_M1MG_NBGROUP:SRING=6 \
                                              -DMOOD:BOOL=ON -DMOOD_MAX_DEGREE:STRING="${MOOD}" \
                                              -DMOOD_PAD:BOOL="${MOOD_PAD}" -DMOOD_DMP:BOOL="${MOOD_DMP}" -DMOOD_ED:BOOL="${MOOD_ED}" \
                                              -DMOOD_U2:BOOL="${MOOD_U2}" -DMOOD_ENTROPIC:BOOL="${MOOD_ENTROPIC}" -DMOOD_CHECK_POLY_REC:BOOL=OFF \
                                              -DMOOD_UNLIMITED:BOOL="${MOOD_UNLIMITED}" -DMOOD_FULL_DEC:BOOL=OFF \
                                              -DTIME:STRING="${TIME}" \
                                              -DTP_CORREC:BOOL="${TP_CORREC}" -DTP_FLUX:BOOL="${TP_FLUX}" -DTP_CORREC_FULL:BOOL="${TP_CORREC_FULL}" \
                                              -DENABLE_SRC:BOOL="${SRC}" -DFLUX_SRC_CENTER:BOOL=OFF -DNO_AP:BOOL="${NO_AP}" \
                                              -DMESH="${MESH}" -DMESH_RCM:BOOL="${RCM}" -DMESH_PERIODIC:BOOL=ON \
                                              -DDLP_NB_PT_INTERP:STRING=3 \
                                              -DENABLE_OMP:BOOL="${OMP}" -DENABLE_LAPACK:BOOL=ON \
                                              -DIO_ENABLE_DUMP:BOOL="${DUMP}" -DENABLE_HDF5:BOOL=OFF -DENABLE_NC:BOOL=OFF \
                                              -DIO_ENABLE_VTK:BOOL="${VTK}"
                                            make "${MAKEFLAGS}" VERBOSE=1 DESTDIR="${BUILDDIR}" \
                                              &> "${logfile}" \
                                              || echo -e "${red}failed${reset} see ${logfile}"
                                            cd "${TESTDIR}" || exit 1
                                          else
                                            make "${MAKEFLAGS}" \
                                              PHY="${PHY}" FLUX="${FLUX}" \
                                              PHY_EULER_ISENTROPIC=0 \
                                              MOOD="${MOOD}"  \
                                              MOOD_UNLIMITED="${MOOD_UNLIMITED}" MOOD_PAD="${MOOD_PAD}" \
                                              MOOD_DMP="${MOOD_DMP}" MOOD_ED="${MOOD_ED}" \
                                              MOOD_U2="${MOOD_U2}" MOOD_ENTROPIC="${MOOD_ENTROPIC}" \
                                              MOOD_FULL_DEC=0 MOOD_CHECK_POLY_REC=0 \
                                              TIME="${TIME}" \
                                              TP_FLUX="${TP_FLUX}" TP_CORREC="${TP_CORREC}" TP_CORREC_FULL="${TP_CORREC_FULL}" \
                                              SRC="${SRC}" FLUX_SRC_CENTER=0 NO_AP="${NO_AP}" \
                                              MESH="${MESH}" RCM="${RCM}" PERIODIC=1 \
                                              DLP_NB_PT_INTERP=3 \
                                              OMP="${OMP}" DEBUG=1 \
                                              OMPP=0 SCALASCA=0 PROFILING=0 COVERAGE=0 \
                                              DUMP="${DUMP}" VTK="${VTK}" \
                                              DESTDIR="${TESTDIR}" \
                                              &> "${logfile}" \
                                              || echo -e "${red}failed${reset} see ${logfile}"
                                          fi

                                        else
                                          # Stop on failure and print log on stdout
                                          if hash cmake 2>/dev/null
                                          then
                                            mkdir -p -- "${TESTDIR}/${BUILDDIR}"
                                            cd "${TESTDIR}/${BUILDDIR}" || exit 1
                                            cmake "${ROOTDIR}" -DCMAKE_BUILD_TYPE:STRING=Debug \
                                              -DPHY:STRING="${PHY}" -DFLUX:STRING="${FLUX}" \
                                              -DPHY_EULER:STRING="IDEAL_GAS" -DPHY_M1MG_NBGROUP:SRING=6 \
                                              -DMOOD:BOOL=ON -DMOOD_MAX_DEGREE:STRING="${MOOD}" \
                                              -DMOOD_PAD:BOOL="${MOOD_PAD}" -DMOOD_DMP:BOOL="${MOOD_DMP}" -DMOOD_ED:BOOL="${MOOD_ED}" \
                                              -DMOOD_U2:BOOL="${MOOD_U2}" -DMOOD_ENTROPIC:BOOL="${MOOD_ENTROPIC}" -DMOOD_CHECK_POLY_REC:BOOL=OFF \
                                              -DMOOD_UNLIMITED:BOOL="${MOOD_UNLIMITED}" -DMOOD_FULL_DEC:BOOL=OFF \
                                              -DTIME:STRING="${TIME}" \
                                              -DTP_CORREC:BOOL="${TP_CORREC}" -DTP_FLUX:BOOL="${TP_FLUX}" -DTP_CORREC_FULL:BOOL="${TP_CORREC_FULL}" \
                                              -DENABLE_SRC:BOOL="${SRC}" -DFLUX_SRC_CENTER:BOOL=OFF -DNO_AP:BOOL="${NO_AP}" \
                                              -DMESH="${MESH}" -DMESH_RCM:BOOL="${RCM}" -DMESH_PERIODIC:BOOL=ON \
                                              -DDLP_NB_PT_INTERP:STRING=3 \
                                              -DENABLE_OMP:BOOL="${OMP}" -DENABLE_LAPACK:BOOL=ON \
                                              -DIO_ENABLE_DUMP:BOOL="${DUMP}" -DENABLE_HDF5:BOOL=OFF -DENABLE_NC:BOOL=OFF \
                                              -DIO_ENABLE_VTK:BOOL="${VTK}"
                                            make "${MAKEFLAGS}" DESTDIR="${BUILDDIR}" \
                                              || exit 1
                                            cd "${TESTDIR}" || exit 1
                                          else
                                            make "${MAKEFLAGS}" \
                                              PHY="${PHY}" FLUX="${FLUX}" \
                                              PHY_EULER_ISENTROPIC=0 \
                                              MOOD="${MOOD}"  \
                                              MOOD_UNLIMITED="${MOOD_UNLIMITED}" MOOD_PAD="${MOOD_PAD}" \
                                              MOOD_DMP="${MOOD_DMP}" MOOD_ED="${MOOD_ED}" \
                                              MOOD_U2="${MOOD_U2}" MOOD_ENTROPIC="${MOOD_ENTROPIC}" \
                                              MOOD_FULL_DEC=0 MOOD_CHECK_POLY_REC=0 \
                                              TIME="${TIME}" \
                                              TP_FLUX="${TP_FLUX}" TP_CORREC="${TP_CORREC}" TP_CORREC_FULL="${TP_CORREC_FULL}" \
                                              SRC="${SRC}" NO_AP="${NO_AP}" \
                                              MESH="${MESH}" RCM="${RCM}" PERIODIC=1 \
                                              DLP_NB_PT_INTERP=3 \
                                              OMP="${OMP}" DEBUG=1 \
                                              OMPP=0 SCALASCA=0 PROFILING=0 COVERAGE=0 \
                                              DUMP="${DUMP}" VTK="${VTK}" \
                                              DESTDIR="${TESTDIR}" || exit 1
                                          fi

                                        fi
                                        NbTests="$((NbTests + 1))"

                                        rm -r -- "${TESTDIR}"/build*

                                      done
                                    done
                                  done
                                done
                              done
                            done
                          done
                        done
                      done
                    done
                  done
                done
              done
            done
          done
        done
      done
    done
  done
done

echo "All tests done"

exit 0
