!> Some parameters for the I/O
!>
!> @author Florian Blachère
!>
!> @copyright © 2017-2018 Université de technologie de Troyes,
!> @copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
!> @copyright © 2013-2018 CNRS,
!> @copyright © 2013-2016 Université de Nantes.
!> @copyright This program is free software: you can redistribute it and/or modify
!> it under the terms of the GNU General Public License as published by
!> the Free Software Foundation, either version 3 of the License, or
!> (at your option) any later version.
!>
!> @copyright This program is distributed in the hope that it will be useful,
!> but WITHOUT ANY WARRANTY; without even the implied warranty of
!> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!> GNU General Public License for more details.
!>
!> @copyright You should have received a copy of the GNU General Public License
!> along with this program.  If not, see <http://www.gnu.org/licenses/>.
module type_IO

  use mod_numerics, only: MaxLenStr
  use, INTRINSIC :: ISO_FORTRAN_ENV, only: OUTPUT_UNIT, ERROR_UNIT

  implicit none

  !> Outputs on screen are done every \c out_screen %
  integer, parameter :: out_screen = 1
  !> Outputs on disk (VTK files) are done every \c out_disk %
  integer, parameter :: out_disk = 5
#if defined(_IO_ENABLE_DUMP_)
  !> Dumps are done every \c dump_disk %
  integer, parameter :: dump_disk = 5
#endif /* _IO_ENABLE_DUMP_ */

#if defined(_IO_ENABLE_VTK_)
  !> Number of outputs to the disk
  integer, save :: n_out_disk
#endif /* _IO_ENABLE_VTK_ */
  !> Number of outputs to the screen (log)
  integer, save :: n_out_screen
#if defined(_IO_ENABLE_DUMP_)
  !> Number of dumps (for restart file)
  integer  :: n_dump_disk
#endif /* _IO_ENABLE_DUMP_ */

  !> Standard output (6)
  integer, parameter :: StdOut = OUTPUT_UNIT
  !> Standard error (0)
  integer, parameter :: StdErr = ERROR_UNIT

  !> Format definition for reals
#if defined(_IO_ALL_DIGITS_)
  ! to get all digits from double
  character(len=9), parameter :: FMT_D = 'ES24.16E3'
#else
  ! to write '-1.2345E-123'
  character(len=8), parameter :: FMT_D = 'ES12.4E3'
#endif

  !> Format definition for integer
  character(len=2), parameter :: FMT_I = 'I9'

  !> Path separator
#if defined(__WIN32__) || defined(__WIN32) || defined(WIN32)
  character(len=1), parameter :: FS = char(92) ! Should be equal to \ (see ichar)
#else
  character(len=1), parameter :: FS = '/'
#endif

  !> Separator strings
  character(len=55), dimension(5), parameter :: separator = [&
    '=======================================================',&
    '-------------------------------------------------------',&
    '.......................................................',&
    '-------------------------                              ',&
    '-------                                                '&
    ]

  !> Output directory for the current test case.
  character(len=MaxLenStr) :: Output_Directory

end module type_IO
