!> Definition of the escape characters to print colors in terminal.
!> @remarks from https://wiki.archlinux.org/index.php/Color_Bash_Prompt#List_of_colors_for_prompt_and_Bash
!> @remarks and http://stackoverflow.com/questions/6402700/coloured-terminal-output-from-fortran
!>
!> @author Florian Blachère
!>
!> @copyright © 2017-2018 Université de technologie de Troyes,
!> @copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
!> @copyright © 2013-2018 CNRS,
!> @copyright © 2013-2016 Université de Nantes.
!> @copyright This program is free software: you can redistribute it and/or modify
!> it under the terms of the GNU General Public License as published by
!> the Free Software Foundation, either version 3 of the License, or
!> (at your option) any later version.
!>
!> @copyright This program is distributed in the hope that it will be useful,
!> but WITHOUT ANY WARRANTY; without even the implied warranty of
!> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!> GNU General Public License for more details.
!>
!> @copyright You should have received a copy of the GNU General Public License
!> along with this program.  If not, see <http://www.gnu.org/licenses/>.
module type_colors

  implicit none

  !> Escape character \c '\\e'
  character(len=1), parameter :: esc_char = ACHAR(27)

  !> Colors definitions
  type colors
    character(len=7) :: txtblk = esc_char//'[0;30m' !< Regular Black
    character(len=7) :: txtred = esc_char//'[0;31m' !< Regular Red
    character(len=7) :: txtgrn = esc_char//'[0;32m' !< Regular Green
    character(len=7) :: txtylw = esc_char//'[0;33m' !< Regular Yellow
    character(len=7) :: txtblu = esc_char//'[0;34m' !< Regular Blue
    character(len=7) :: txtpur = esc_char//'[0;35m' !< Regular Purple
    character(len=7) :: txtcyn = esc_char//'[0;36m' !< Regular Cyan
    character(len=7) :: txtwht = esc_char//'[0;37m' !< Regular White
    character(len=7) :: bldblk = esc_char//'[1;30m' !< Bold Black
    character(len=7) :: bldred = esc_char//'[1;31m' !< Bold Red
    character(len=7) :: bldgrn = esc_char//'[1;32m' !< Bold Green
    character(len=7) :: bldylw = esc_char//'[1;33m' !< Bold Yellow
    character(len=7) :: bldblu = esc_char//'[1;34m' !< Bold Blue
    character(len=7) :: bldpur = esc_char//'[1;35m' !< Bold Purple
    character(len=7) :: bldcyn = esc_char//'[1;36m' !< Bold Cyan
    character(len=7) :: bldwht = esc_char//'[1;37m' !< Bold White
    character(len=7) :: unkblk = esc_char//'[4;30m' !< Underline Black
    character(len=7) :: undred = esc_char//'[4;31m' !< Underline Red
    character(len=7) :: undgrn = esc_char//'[4;32m' !< Underline Green
    character(len=7) :: undylw = esc_char//'[4;33m' !< Underline Yellow
    character(len=7) :: undblu = esc_char//'[4;34m' !< Underline Blue
    character(len=7) :: undpur = esc_char//'[4;35m' !< Underline Purple
    character(len=7) :: undcyn = esc_char//'[4;36m' !< Underline Cyan
    character(len=7) :: undwht = esc_char//'[4;37m' !< Underline White
    character(len=5) :: bakblk = esc_char//'[40m'   !< Background Black
    character(len=5) :: bakred = esc_char//'[41m'   !< Background Red
    character(len=5) :: bakgrn = esc_char//'[42m'   !< Background Green
    character(len=5) :: bakylw = esc_char//'[43m'   !< Background Yellow
    character(len=5) :: bakblu = esc_char//'[44m'   !< Background Blue
    character(len=5) :: bakpur = esc_char//'[45m'   !< Background Purple
    character(len=5) :: bakcyn = esc_char//'[46m'   !< Background Cyan
    character(len=5) :: bakwht = esc_char//'[47m'   !< Background White
    character(len=4) :: txtrst = esc_char//'[0m'    !< Background Text Reset
  end type colors

  !> Variable containing all the previously defined colors.
  type(colors), save :: color

end module type_colors
