!> Physics of the Shallow Water equations \cite StV_1871 .
!>
!> \f[\left\{\begin{array}{rcl}
!>   \partial_t h   + \partial_x h u            + \partial_y h v            & = & 0 \\
!>   \partial_t h u + \partial_x (h u^2 + p(h)) + \partial_y h uv           & = & - \frac{\kappa}{h^\eta} h u \\
!>   \partial_t h v + \partial_x  h u v         + \partial_y (h v^2 + p(h)) & = & - \frac{\kappa}{h^\eta} h u \\
!> \end{array}\right.\f]
!> with:
!> * \f$ h \f$: height,
!> * \f$ \vec{V} =(u,v) \f$: speed of the fluid,
!> * \f$ g \f$: gravity constannt,
!> * \f$ \kappa \f$: friction,
!> * \f$ p \f$: pressure,
!> * and the speed of the sound in the fluid is defined as: \f$c = \sqrt{gh}\f$.
!>
!> \cite StV_1871
!>
!> @author Florian Blachère
!>
!> @copyright © 2017-2018 Université de technologie de Troyes,
!> @copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
!> @copyright © 2013-2018 CNRS,
!> @copyright © 2013-2016 Université de Nantes.
!> @copyright This program is free software: you can redistribute it and/or modify
!> it under the terms of the GNU General Public License as published by
!> the Free Software Foundation, either version 3 of the License, or
!> (at your option) any later version.
!>
!> @copyright This program is distributed in the hope that it will be useful,
!> but WITHOUT ANY WARRANTY; without even the implied warranty of
!> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!> GNU General Public License for more details.
!>
!> @copyright You should have received a copy of the GNU General Public License
!> along with this program.  If not, see <http://www.gnu.org/licenses/>.
module mod_physics_SW_var
#if defined(_PHY_SW_)

  use mod_numerics, only: PR, MaxLenStr

  implicit none

  private
  public NbEq, NbEq_str, T_init, T_end
  public len_model, len_primitive, len_conservative
  public model_name, primitive_name, conservative_name, test_case

  public gravity, kappa, eta_friction

  integer, parameter                  :: len_model = 2
  !> Name of the model
  character(len=len_model), parameter :: model_name = 'SW'
  !> Number of equations (unknowns) in the model
  integer, parameter                  :: NbEq = 3
  !> Number of equations (unknowns) in the model, in a string
  character(len=1), parameter :: NbEq_str = '3'

  !> Intial time of the simulation
  real(PR), save :: T_init
  !> Final time of the simulation
  real(PR), save :: T_end

  !> Gravity constant
  real(PR), parameter :: gravity = 9.81_PR
  !> Friction coefficient
  real(PR), parameter :: kappa = 5.E1_PR
  !> Friction parameter
  real(PR), parameter :: eta_friction = 2._PR/3._PR

  integer, parameter :: len_primitive = 1
  integer, parameter :: len_conservative = 3

  !> Primitive variables used in the model
  character(len=len_primitive),    dimension(NbEq), parameter :: primitive_name = ['h', 'u', 'v']
  !> Conservative variables used in the model
  character(len=len_conservative), dimension(NbEq), parameter :: conservative_name = ['h  ', 'h_u', 'h_v']

  !> Name of the test case
  character(len=MaxLenStr), save :: test_case

#endif /* _PHY_SW_ */
end module mod_physics_SW_var
