!> Physics module for the Telegraph (or Goldstein-Taylor) equations
!>
!> \f[\left\{\begin{array}{rcl}
!>   \partial_t u + \text{div}(\vec{a} u) &=& \sigma (v - u) \\
!>   \partial_t v - \text{div}(\vec{a} v) &=& \sigma (u - v)
!> \end{array}\right.\f]
!>
!> @author Florian Blachère
!>
!> @copyright © 2017-2018 Université de technologie de Troyes,
!> @copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
!> @copyright © 2013-2018 CNRS,
!> @copyright © 2013-2016 Université de Nantes.
!> @copyright This program is free software: you can redistribute it and/or modify
!> it under the terms of the GNU General Public License as published by
!> the Free Software Foundation, either version 3 of the License, or
!> (at your option) any later version.
!>
!> @copyright This program is distributed in the hope that it will be useful,
!> but WITHOUT ANY WARRANTY; without even the implied warranty of
!> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!> GNU General Public License for more details.
!>
!> @copyright You should have received a copy of the GNU General Public License
!> along with this program.  If not, see <http://www.gnu.org/licenses/>.
module mod_physics_Telegraph_var
#if defined (_PHY_TELE_)

  use mod_numerics, only: PR, MaxLenStr

  implicit none

  private
  public NbEq, NbEq_str, T_init, T_end
  public len_model, len_primitive, len_conservative
  public model_name, primitive_name, conservative_name, test_case

  public sigma, speed_tele

  integer, parameter :: len_model = 9
  !> Name of the model
  character(len=len_model), parameter :: model_name = 'Telegraph'
  !> Number of equations (unknowns) in the model
  integer, parameter :: NbEq = 2
  !> Number of equations (unknowns) in the model, in a string
  character(len=1), parameter :: NbEq_str = '2'

  !> Asymptotic parameter
  real(PR), parameter :: sigma = 1.E3_PR
  !> Speed
  real(PR), save :: speed_tele

  !> Intial time of the simulation
  real(PR), save :: T_init
  !> Final time of the simulation
  real(PR), parameter :: T_end = 10.0_PR

  integer, parameter :: len_primitive    = 1
  integer, parameter :: len_conservative = 1

  !> Primitive variables used in the model
  character(len=len_primitive),    dimension(NbEq), parameter :: primitive_name    = ['u', 'v']
  !> Conservative variables used in the model
  character(len=len_conservative), dimension(NbEq), parameter :: conservative_name = ['u', 'v']

  !> Name of the test case
  character(len=MaxLenStr), save :: test_case

#endif /* _PHY_TELE_ */
end module mod_physics_Telegraph_var
