!> Tools to create directories
!>
!> @author Florian Blachère
!>
!> @copyright © 2017-2018 Université de technologie de Troyes,
!> @copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
!> @copyright © 2013-2018 CNRS,
!> @copyright © 2013-2016 Université de Nantes.
!> @copyright This program is free software: you can redistribute it and/or modify
!> it under the terms of the GNU General Public License as published by
!> the Free Software Foundation, either version 3 of the License, or
!> (at your option) any later version.
!>
!> @copyright This program is distributed in the hope that it will be useful,
!> but WITHOUT ANY WARRANTY; without even the implied warranty of
!> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!> GNU General Public License for more details.
!>
!> @copyright You should have received a copy of the GNU General Public License
!> along with this program.  If not, see <http://www.gnu.org/licenses/>.
module mod_directory

  use type_IO, only: FS
#if defined(_FC_INTEL_)
  use IFPORT, only: access
#endif /* defined(_FC_INTEL_) */
  use mod_execute_command, only: ExecuteCommand

  implicit none

  private
  public CreateOutputDirectory

contains
  !--------------------------------------------------------------------!
  !> @brief
  !> Create the output directory
  !>
  !> @author Florian Blachère
  !--------------------------------------------------------------------!
  subroutine CreateOutputDirectory(Output_Directory)
    character(len=*), intent(in) :: Output_Directory

#if defined(_FC_PGI_)
    integer :: access
#endif

    ! Trick to test if the folder is writable
    if( access(Output_Directory//FS//'.', 'w') /= -1 ) then
#if defined(__WIN32__) || defined(__WIN32) || defined(WIN32)
      call ExecuteCommand('mkdir '//trim(Output_Directory))
#else
      call ExecuteCommand('mkdir -p '//trim(Output_Directory))
#endif
    endif

  end subroutine CreateOutputDirectory

end module mod_directory
