!> Display memory usage
!>
!> @copyright © 2017-2018 Université de technologie de Troyes,
!> @copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
!> @copyright © 2013-2018 CNRS,
!> @copyright © 2013-2016 Université de Nantes.
!> @copyright This program is free software: you can redistribute it and/or modify
!> it under the terms of the GNU General Public License as published by
!> the Free Software Foundation, either version 3 of the License, or
!> (at your option) any later version.
!>
!> @copyright This program is distributed in the hope that it will be useful,
!> but WITHOUT ANY WARRANTY; without even the implied warranty of
!> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!> GNU General Public License for more details.
!>
!> @copyright You should have received a copy of the GNU General Public License
!> along with this program.  If not, see <http://www.gnu.org/licenses/>.

module mod_display_memory
#if defined(__unix__) || defined(__unix) || defined(unix)

  use mod_numerics, only: MaxLenStr
  use type_IO, only: StdOut, StdErr, FS
  use mod_execute_command, only: ExecuteCommand
#if defined(_FC_INTEL_)
  use IFPORT, only: GETPID
#endif /* defined(_FC_INTEL_) */

  implicit none

  private
  public DisplayMemoryUsage

contains
  !--------------------------------------------------------------------!
  !> @brief
  !> Display current memory usage
  !>
  !> @author Florian Blachère
  !--------------------------------------------------------------------!
  subroutine DisplayMemoryUsage()
    character(len=MaxLenStr) :: PID_str, output
    integer :: PID
#if defined(_FC_PGI_)
    integer :: GETPID
#endif

    ! Get the PID of the running process
    PID = GETPID()
    write(PID_str,'(I9)') PID

    call ExecuteCommand('echo '//trim(PID_str)//"| xargs -I {} cat /proc/{}/status 2>/dev/null | grep VmRSS | cut -d: -f2 | tr -d '[[:space:]]'",output)

    ! Write the result
    write(StdOut,'(A,A)') 'Memory usage             : ', trim(output)
  end subroutine DisplayMemoryUsage

#endif /* defined(__unix__) || defined(__unix) || defined(unix) */
end module mod_display_memory
