!> Tools for PBS
!>
!> @author Florian Blachère
!>
!> @copyright © 2017-2018 Université de technologie de Troyes,
!> @copyright © 2016-2017 Université de Versailles Saint-Quentin-en-Yvelines,
!> @copyright © 2013-2018 CNRS,
!> @copyright © 2013-2016 Université de Nantes.
!> @copyright This program is free software: you can redistribute it and/or modify
!> it under the terms of the GNU General Public License as published by
!> the Free Software Foundation, either version 3 of the License, or
!> (at your option) any later version.
!>
!> @copyright This program is distributed in the hope that it will be useful,
!> but WITHOUT ANY WARRANTY; without even the implied warranty of
!> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!> GNU General Public License for more details.
!>
!> @copyright You should have received a copy of the GNU General Public License
!> along with this program.  If not, see <http://www.gnu.org/licenses/>.
module mod_tools_pbs

  use mod_numerics, only: MaxLenStr
  use type_IO, only: StdOut, StdErr

  implicit none

  private
  public DisplayPBS_status

contains
  !--------------------------------------------------------------------!
  !> @brief
  !> Display PBS status
  !>
  !> @author Florian Blachère
  !--------------------------------------------------------------------!
  subroutine DisplayPBS_status()
      integer, parameter :: len_PBS_var = 13
      integer, parameter :: NbPBS_var = 6
      character(len=len_PBS_var), dimension(NbPBS_var), parameter :: PBS_var = [&
          'PBS_JOBID    ', &
          'PBS_JOBNAME  ', &
          'PBS_QUEUE    ', &
          'PBS_O_HOST   ', &
          'PBS_O_QUEUE  ', &
          'PBS_O_WORKDIR' &
          ]
      character(len=MaxLenStr) :: str
      integer :: len_arg, IERR, idx

      do idx = 1, NbPBS_var
        call get_environment_variable(trim(adjustl(PBS_var(idx))), str, len_arg, IERR, .TRUE.)
        select case (IERR)
        case (1)
          write(StdOut,'(A)') PBS_var(idx)//' does not exist'
        case (0)
          !if (len_arg > 0) then
          write(StdOut,'(A,A)') PBS_var(idx)//" = ", trim(str)
          !end if
        case (- 1)
          write(StdOut,'(A,I8,A,I8)') ' length = ', len_arg, ' truncated to', MaxLenStr
          write(StdErr,'(A)')"======================================================="
          flush(StdErr)
          STOP
        case default !(>= 2)
          write(StdOut,'(A,I1)') 'get_environment_variable failed: IERR = ', IERR
          write(StdErr,'(A)')"======================================================="
          flush(StdErr)
          STOP
        end select
      end do

      write(StdOut,'(A)')"......................................................."

  end subroutine DisplayPBS_status

end module mod_tools_pbs
